# How to apply SPARQL queries against a KB using NIST-SPARQL-Evaluation docker (version 2.3)

(Last modified: May 24th, 2019)

## Introduction

This document describes how to apply SPARQL queries against a KB using NIST-SPARQL-Evaluation docker as a standalone self-contained utility.

This docker comes pre-packaged with the following:

1. Free version of GraphDB (version 8.9.0) -- `./NIST-SPARQL-Evaluation/AUX-data/graphdb-free-8.9.0-dist.zip`

   NIST-SPARQL-Evaluation docker uses GraphDB as the triple-store for storing (and applying queries against) the KB represented in AIDA Interchange Format. NOTE that the free version comes with the limitation of being able to run no more than two queries in parallel.

   In order to make use of one of GraphDBs paid version place the installer in AUX-data and by supplying the corresponding `GRAPHDB_VERSION` and `GRAPHDB_EDITION` to make when invoking `make build`.

2. Local repository configuration file -- `./NIST-SPARQL-Evaluation/AUX-data/Local-config.ttl`

   This file is used to create the default repository (named `Local`) for storing the KB.

3. Custom function: GetSpan (version 1.0.1b) -- `./NIST-SPARQL-Evaluation/AUX-data/rdf4j-function-getspan-1.0.1b-SNAPSHOT.jar`

   This custom function is used to obtain a standard representation of mention spans across different modalities. This custom function was developed by NIST, and is currently under review by Next Century. After the review, the code will become part of the master branch of VERDI-SPARQL-Evaluation git repository. At that point, you will find a README file inside `custom-functions/rdf4j/rdf4j-function-getspan`. Please refer to this README file (when made available) for more details.

4. Custom function: SuperTypeOf (version 1.0.1b) -- `./NIST-SPARQL-Evaluation/AUX-data/rdf4j-function-supertypeof-1.0.1b-SNAPSHOT.jar`

   This custom function is used for applying filter for type matches. This custom function was developed by Next Century, and was modified by NIST. These modifications are currently under review by Next Century. After the review, the modified code will become part of the master branch of VERDI-SPARQL-Evaluation git repository, at which point, you will find a README file inside `custom-functions/rdf4j/rdf4j-function-supertypeof`. Please refer to this README file (when made available) for more details.

5. Custom function: MemberOf (version 1.0.0b) -- `./NIST-SPARQL-Evaluation/AUX-data/rdf4j-function-memberof-1.0.0b-SNAPSHOT.jar`

    This custom function is used for applying filter for reference KBID matches in the KB against a set of KBIDs provided by LDC. This custom function was developed by NIST, and is currently under review by Next Century. After the review, the code will become part of the master branch of VERDI-SPARQL-Evaluation git repository. At that point, you will find a README file inside `custom-functions/rdf4j/rdf4j-function-memberof`. Please refer to this README file (when made available) for more details.

6. Custom function: ValidEdge (version 1.0.0b) -- `./NIST-SPARQL-Evaluation/AUX-data/rdf4j-function-validedge-1.0.0b-SNAPSHOT.jar`

    This custom function is used for applying filter in TA3 SPARQL queries that check if the subject type is consistent with the edge label. This is determined by splitting the edge_type into two tokens, using underscore (`_`) as the delimiter, and checking if the first token matches strictly with the subject type. This custom function was developed by NIST, and is currently under review by Next Century. After the review, the code will become part of the master branch of VERDI-SPARQL-Evaluation git repository. At that point, you will find a README file inside `custom-functions/rdf4j/rdf4j-function-validedge`. Please refer to this README file (when made available) for more details.

7. VERDI-SPARQL-Evaluation (version 1.0.0) -- `./NIST-SPARQL-Evaluation/AUX-data/sparql-evaluation-1.0.0-SNAPSHOT-all.jar`

   The NIST-SPARQL-Evaluation docker is a wrapper written around the version 1.0.0 of VERDI-SPARQL-Evaluation tool developed by Next Century.

   Source code of VERDI-SPARQL-Evaluation can be found at: `https://github.com/NextCenturyCorporation/VERDI-SPARQL-Evaluation`

8. Configuration file required by VERDI-SPARQL-Evaluation -- `./NIST-SPARQL-Evaluation/AUX-data/Local-config.properties`

   This configuration file specifies the RDF4J configurations used for applying queries against the `Local` repository.

9. Dockerfile -- `./NIST-SPARQL-Evaluation/Dockerfile`

   This file contains the specifications of the NIST-SPARQL-Evaluation docker.

10. Makefile -- `./NIST-SPARQL-Evaluation/Makefile`

   The Makefile for building and running the docker.

11. scripts/Makefile -- `./NIST-SPARQL-Evaluation/scripts/Makefile`

   The Makefile for running queries against a KB using `VERDI-SPARQL-Evaluation`.

12. Example data -- `./M18-data`

   The docker comes with the following:

   (a) Two runs (or submissions) -- `./M18-data/runs`

   (b) Four queries -- `./M18-data/queries`

   (c) Example output -- `./M18-data/example-output`

## How to build the docker

#### Using the default (free) version of GraphDB

In order to build the docker using the default (free) version of GraphDB, run the following command(s):

~~~
cd NIST-SPARQL-Evaluation
make build
~~~

#### Using paid version of GraphDB

In order to build the docker with either a paid version of GraphDB or a version different from the one used by default, you would need to

1. Download the paid version of GraphDB `graphdb-[otheredition]-[otherversion]-dist.zip` and place it inside `NIST-SPARQL-Evaluation/AUX-data/`, and

2. Run the following command(s):

~~~
cd NIST-SPARQL-Evaluation
make build GRAPHDB_EDITION=otheredition GRAPHDB_VERSION=otherversion
~~~

## How to run the docker

#### Running example queries against example run(s)

In order to apply example queries over example runs (or submissions), run the following commands:

~~~
make run-all
~~~

You can find the output at `./M18-data/output`. In order to verify that the docker worked as expected, you may want to compare the content of `./M18-data/output` with `./M18-data/example-output`.

#### Running example queries against your own run

In order to run the example queries against your own run (or submission), you may run the `make run` command, as shown below, and supply the following:

Parameter | Description
--- | ---
`HOST_RUN_DIR` | the directory containing the task1 KBs or task2 KB
`HOST_QUERIES_DIR` | the directory containing SPARQL query files (\*.rq files) for a given query type
`HOST_OUTPUT_DIR` | the directory where output will be written

~~~
make run \
      HOST_RUN_DIR=/path/to/your/run \
      HOST_QUERIES_DIR=/path/to/queries \
      HOST_OUTPUT_DIR=/path/to/output \
~~~

## Revision history
#### version 1.0
  * Initial version containing only example queries

#### version 2.0
  * More example queries added

#### version 2.1
  * Example KBs modified to fix validation errors

#### version 2.2
  * In order to support matching against ambiguous nodes that were split in reference KB
    * Custom function `memberOf` added
    * TA2 queries modified to make use of custom function memberOf
  * Example KBs modified to fix validation errors

#### version 2.3
  * TA1 and TA2 graph queries had a bug in where SPARQL was looking up for confidence value of justification of an argument assertion. Originally, the confidence associated with an argument assertion was mistakenly extracted and used as the confidence of the justification of the argument assertion. The queries that come along with this version are generated from the new template of queries.
  * Example TA3 hypothesis added
  * TA3 graph SPARQL query added
  * Custom function `validEdge` added. This custom function is only used in TA3 graph SPARQL query.
